@@ -84,17 +84,36 @@ def dinuc_count(seq):
     Calculate dinucleotide composition of a sequence.
     We consider the IUPAC alphabet here.
     """
-    return map(seq.count, IUPAC_DINUC)
+    return map(seq.count_overlap, IUPAC_DINUC)
+
+
+def above_threshold(the_list, threshold):
+    """
+    Returns the percentage of values strictly above the given threshold.
+    """
+    import numpy as np
+    return np.count_nonzero(np.array(the_list) < threshold) / len(the_list)
+    # return(iterlen(val for val in the_list if val < threshold) /
+    # len(the_list))
 
 
 def power_div(fg_dist, bg_dist, lambda_="pearson"):
     """
     Compute the power divergence between two distributions.
+    Need to test for statistical power first. Given scipy documentation:
+        This test is invalid when the observed or expected frequencies in each
+        category are too small. A typical rule is that all of the observed and
+        expected frequencies should be at least 5.
     """
+    import numpy as np
+    # Removing 0-values in the expectation
+    f_exp = np.array(fg_dist)
+    f_obs = np.array(bg_dist)[f_exp != 0].tolist()
+    f_exp = f_exp[f_exp != 0].tolist()
+    if ((above_threshold(f_exp, 5) > 0.2) or
+       (above_threshold(f_obs, 5) > 0.2)):
+        return None, -1
     from scipy.stats import power_divergence
-    # We add 1 to all values to avoid division by 0
-    f_exp = [val + 1 for val in fg_dist]
-    f_obs = [val + 1 for val in bg_dist]
     return power_divergence(f_exp=f_exp, f_obs=f_obs, lambda_=lambda_)
 
 
@@ -106,6 +125,31 @@ def single_value(the_array):
     return len(np.unique(the_array)) == 1
 
 
+def QC_info(fg_hist, bg_hist, out_filename):
+    """
+    Compute QC metrics and print info to out_filename.
+    """
+    mean_abs_error = mae(fg_hist, bg_hist)
+    chi_stat, chi_pval = power_div(fg_hist, bg_hist)
+    gof_stat, gof_pval = power_div(fg_hist, bg_hist, "cressie-read")
+    with open(out_filename, 'w') as stream:
+        if sum(fg_hist) < 1000 or sum(bg_hist) < 1000:
+            stream.write("QC tests cannot be ")
+            stream.write("computed due to a small number of samples ")
+            stream.write("(less than 1000).\n")
+        elif chi_pval == -1 or gof_pval == -1:
+            stream.write("QC tests cannot be ")
+            stream.write("computed due to a large number of values ")
+            stream.write("with low frequencies (more than ")
+            stream.write("20% of values <=5).\n")
+        else:
+            stream.write("mean_absolute_error\t%f\n" % mean_abs_error)
+            stream.write("chi-square(statistic, pvalue)\t(%f, %f)\n" %
+                         (chi_stat, chi_pval))
+            stream.write("cressie-read goodness_of_fit(statistic, pvalue)")
+            stream.write("\t(%f, %f)\n" % (gof_stat, gof_pval))
+
+
 def make_gc_plot(fg_gc, bg_gc, plot_filename):
     """
     Compute the density GC composition plots for background and input.
@@ -133,25 +177,14 @@ def make_gc_plot(fg_gc, bg_gc, plot_filename):
                         label='generated')
     plt.legend()
     plot.set(xlabel="%GC", ylabel=ylab)
-    mean_abs_error = mae(fg_hist, bg_hist)
-    chi_stat, chi_pval = power_div(fg_hist, bg_hist)
-    gof_stat, gof_pval = power_div(fg_hist, bg_hist, "cressie-read")
-    the_text = "mean absolute error: %.2f; " % mean_abs_error
-    the_text += "chisquare: %.2f, p-val: %.2f; " % (chi_stat, chi_pval)
-    the_text += "cressie-read: %.2f, p-val: %.2f" % (gof_stat, gof_pval)
-    plt.figtext(.5, .97, the_text, ha='center', va='center')
-    with open("{0}_gc_plot_stats.txt".format(plot_filename), 'w') as stream:
-        stream.write("mean_absolute_error\t%f\n" % mean_abs_error)
-        stream.write("chi-square\t%f,%f\n" % (chi_stat, chi_pval))
-        stream.write("goodness_of_fit(cressie-read)\t%f,%f\n" % (gof_stat,
-                                                                 gof_pval))
     plt.savefig("{0}_gc_plot.png".format(plot_filename))
+    QC_info(fg_hist, bg_hist, "{0}_gc_plot_stats.txt".format(plot_filename))
 
 
 def make_len_plot(fg_len, bg_len, plot_filename):
     """
-    Compute the density length plot for the background, the input and the
-    matching background datasets.
+    Compute the density length plot for the input and the matching background
+    datasets.
     """
     from numpy import histogram
     from numpy import hstack
@@ -176,27 +209,20 @@ def make_len_plot(fg_len, bg_len, plot_filename):
         plt.hist(fg_len, bins=bins, align='right', alpha=.5, label='input')
         plt.hist(bg_len, bins=bins, align='right', alpha=.5, label='generated')
     else:
-        sns.distplot(fg_len, kde=True, kde_kws={'shade': True, 'linewidth': 3},
+        sns.distplot(fg_len, hist=plot_hist, kde=True, kde_kws={'shade': True,
+                                                                'linewidth':
+                                                                3},
                      label='input')
-        sns.distplot(bg_len, kde=True, kde_kws={'shade': True, 'linewidth': 3},
+        sns.distplot(bg_len, hist=plot_hist, kde=True, kde_kws={'shade': True,
+                                                                'linewidth':
+                                                                3},
                      label='generated')
     plt.xlabel('length')
     plt.ylabel(ylab)
     plt.legend()
-    mean_abs_error = mae(fg_hist, bg_hist)
-    chi_stat, chi_pval = power_div(fg_hist, bg_hist)
-    gof_stat, gof_pval = power_div(fg_hist, bg_hist, "cressie-read")
-    the_text = "mean absolute error: %.2f; " % mean_abs_error
-    the_text += "chisquare: %.2f, p-val: %.2f; " % (chi_stat, chi_pval)
-    the_text += "cressie-read: %.2f, p-val: %.2f" % (gof_stat, gof_pval)
-    plt.figtext(.5, .97, the_text, ha='center', va='center')
     basename = "{0}_length_plot".format(plot_filename)
+    QC_info(fg_hist, bg_hist, "{0}_stats.txt".format(basename))
     plt.savefig("{0}.png".format(basename))
-    with open("{0}_stats.txt".format(basename), 'w') as stream:
-        stream.write("mean_absolute_error\t%f\n" % mean_abs_error)
-        stream.write("chi-square\t%f,%f\n" % (chi_stat, chi_pval))
-        stream.write("goodness_of_fit(cressie-read)\t%f,%f\n" % (gof_stat,
-                                                                 gof_pval))
 
 
 def make_dinuc_dico(dinuc_counts):
@@ -220,6 +246,8 @@ def make_dinuc_plot(fg_dinuc, bg_dinuc, plot_filename):
     import matplotlib
     matplotlib.use('Agg')
     import matplotlib.pyplot as plt
+    QC_info(fg_dinuc, bg_dinuc,
+            "{0}_dinuc_plot_stats.txt".format(plot_filename))
     fg_total = sum(fg_dinuc)
     fg_dinuc = [val / fg_total for val in fg_dinuc]
     dico = make_dinuc_dico(fg_dinuc)
@@ -245,19 +273,7 @@ def make_dinuc_plot(fg_dinuc, bg_dinuc, plot_filename):
                  use_gridspec=False, pad=0.2)
     ax2.yaxis.tick_right()
     ax2.tick_params(labelrotation=0)
-    mean_abs_error = mae(fg_dinuc, bg_dinuc)
-    chi_stat, chi_pval = power_div(fg_dinuc, bg_dinuc)
-    gof_stat, gof_pval = power_div(fg_dinuc, bg_dinuc, "cressie-read")
-    the_text = "mean absolute error: %.2f; " % mean_abs_error
-    the_text += "chisquare: %.2f, p-val: %.2f; " % (chi_stat, chi_pval)
-    the_text += "cressie-read: %.2f, p-val: %.2f" % (gof_stat, gof_pval)
-    plt.figtext(.5, .97, the_text, ha='center', va='center')
     plt.savefig("{0}_dinuc_plot.png".format(plot_filename))
-    with open("{0}_dinuc_plot_stats.txt".format(plot_filename), 'w') as stream:
-        stream.write("mean_absolute_error\t%f\n" % mean_abs_error)
-        stream.write("chi-square\t%f,%f\n" % (chi_stat, chi_pval))
-        stream.write("goodness_of_fit(cressie-read)\t%f,%f\n" % (gof_stat,
-                                                                 gof_pval))
 
 
 def make_dinuc_acgt_only_dico(dinuc_counts):
@@ -292,6 +308,12 @@ def make_dinuc_acgt_only_plot(fg_dinuc, bg_dinuc, plot_filename):
     matplotlib.use('Agg')
     import matplotlib.pyplot as plt
     acgt = ['A', 'C', 'G', 'T']
+    fg_dico = make_dinuc_acgt_only_dico(fg_dinuc)
+    fg_dinuc_count = [item for sublist in fg_dico.values() for item in sublist]
+    bg_dico = make_dinuc_acgt_only_dico(bg_dinuc)
+    bg_dinuc_count = [item for sublist in bg_dico.values() for item in sublist]
+    basename = "{0}_dinuc_acgt_only_plot".format(plot_filename)
+    QC_info(fg_dinuc_count, bg_dinuc_count, "{0}_stats.txt".format(basename))
     fg_total = sum(fg_dinuc)
     fg_dinuc = [val / fg_total for val in fg_dinuc]
     fg_dico = make_dinuc_acgt_only_dico(fg_dinuc)
@@ -317,19 +339,4 @@ def make_dinuc_acgt_only_plot(fg_dinuc, bg_dinuc, plot_filename):
                  use_gridspec=False, pad=0.2)
     ax2.yaxis.tick_right()
     ax2.tick_params(labelrotation=0)
-    fg_dinuc = [item for sublist in fg_dico.values() for item in sublist]
-    bg_dinuc = [item for sublist in bg_dico.values() for item in sublist]
-    mean_abs_error = mae(fg_dinuc, bg_dinuc)
-    chi_stat, chi_pval = power_div(fg_dinuc, bg_dinuc)
-    gof_stat, gof_pval = power_div(fg_dinuc, bg_dinuc, "cressie-read")
-    the_text = "mean absolute error: %.2f; " % mean_abs_error
-    the_text += "chisquare: %.2f, p-val: %.2f; " % (chi_stat, chi_pval)
-    the_text += "cressie-read: %.2f, p-val: %.2f" % (gof_stat, gof_pval)
-    plt.figtext(.5, .97, the_text, ha='center', va='center')
-    basename = "{0}_dinuc_acgt_only_plot".format(plot_filename)
     plt.savefig("{0}.png".format(basename))
-    with open("{0}_stats.txt".format(basename), 'w') as stream:
-        stream.write("mean_absolute_error\t%f\n" % mean_abs_error)
-        stream.write("chi-square\t%f,%f\n" % (chi_stat, chi_pval))
-        stream.write("goodness_of_fit(cressie-read)\t%f,%f\n" % (gof_stat,
-                                                                 gof_pval))
